unit ulearningcfcerebros;

{$mode delphi}

interface

uses
  Classes, SysUtils,
  math,
  xmatdefs, utrayectorias_xcdpcf,uclasificador,
  uadministradordeestadoscerebro, matreal,
  utipos_xcf,
  ufxgx,
  uproblema,
  uparsimoniat,
  uneuronas;


function AjusteCFCerebro(
   CF: TAdminEstadosCerebro;
   trayectorias: TTrayectorias;
   parsimonia: TParsimonia;
   learningRate: NReal ): NReal;



implementation


type
  { TCF_fCerebros }

  TCF_fCerebros = class(Tfx)
    CF: TAdminEstadosCerebro;
    trayectorias: TTrayectorias;
    parsimonia: TParsimonia;

    constructor Create(
       CF: TAdminEstadosCerebro; trayectorias: TTrayectorias;
      parsimonia: TParsimonia);

    procedure SetParametrosCerebros( const X: TVectR );
    function f(const X: TVectR): NReal; override;
    procedure acum_g(var grad: TVectR; const X: TVectR); override;

  end;






procedure  Entrenar(
   aCerebro_x: TCerebro;
   trayectorias: TTrayectorias;
   kPaso: integer );
var
  aTrayectoria: TTrayectoria;
  aData: TDAOfRecXCF;
  kCronica: integer;
  ix, ioffset: integer;
  xcf: TRecXCF;
  flg_Convergio: boolean;
  cntIters: integer;

begin
  aData:= utipos_xcf.Crear_data_xcf_0( trayectorias.DimX, trayectorias.nTrayectorias );

  for kCronica:= 0 to trayectorias.nTrayectorias-1 do
  begin
    xcf:= aData[ kCronica ];
    aTrayectoria:= trayectorias.Trayectorias[kCronica];
    for ix:= 0 to aTrayectoria.nr - 1 do
      xcf.X.pon_e( ix + 1, aTrayectoria.puntos[ kPaso ].X.Xr.e(ix+1) );
    ioffset:= aTrayectoria.nr;
    for ix:= 0 to aTrayectoria.nd - 1 do
      xcf.X.pon_e( ioffset + ix + 1, aTrayectoria.puntos[ kPaso ].X.Xd.e(ix+1) );
    xcf.Y.pon_e(1, aTrayectoria.puntos[ kPaso ].CF );
  end;

  aCerebro_x.EscalarEntradas(aData);
  aCerebro_x.EscalarCapaSalida(aData);

  EntrenarCerebro( aCerebro_x, aData, 10000, flg_Convergio, cntIters, -1, -1);
  CalcularYGuardar( 'rn_'+IntToStr(kPaso)+'.xlt', aCerebro_x,  aData);
  free_data_xcf( aData );
end;

{ TCF_fCerebros }

constructor TCF_fCerebros.Create(
   CF: TAdminEstadosCerebro;
   trayectorias: TTrayectorias;
   parsimonia: TParsimonia );
begin
  self.CF:= CF;
  self.trayectorias:= trayectorias;
  self.parsimonia:= parsimonia;
end;

procedure TCF_fCerebros.SetParametrosCerebros(const X: TVectR);
var
  parametros: TVectR;
  aCerebro: TCerebro;

  cerebros: TDAOfCerebros;
  kPaso: integer;

begin
  cerebros:= CF.Cerebros;
  aCerebro:= Cerebros[1];
  parametros:= TVectR.Create_Init( aCerebro.DimParametros );

  for kPaso:= 1 to high( Cerebros ) do
  begin
    aCerebro:= Cerebros[kPaso];

    parsimonia.set_dtk( kPaso );
    parsimonia.fp( parametros, X);
    aCerebro.SetParametros( parametros );

  end;
  parametros.Free;
end;



function TCF_fCerebros.f(const X: TVectR): NReal;
var
  acum_e2: NReal;
  kRec: integer;
  aRec: TPuntoTrayectoria;
  parametros: TVectR;
  aCerebro: TCerebro;

  fDivisor: NReal;
  cerebros: TDAOfCerebros;
  aData: TPuntosTrayectoria;
  kPaso: integer;
  px: TVectR;
  derr: NReal;

begin
  SetParametrosCerebros( X );
  cerebros:= CF.Cerebros;
  aCerebro:= Cerebros[1];

  parametros:= TVectR.Create_Init( aCerebro.DimParametros );
  px:= TVectR.Create_Init( trayectorias.DimX );

  fDivisor:= 1/ ( length( Cerebros ) * trayectorias.nTrayectorias );

  acum_e2 := 0;

  for kPaso:= 1 to high( Cerebros ) do
  begin
    aCerebro:= Cerebros[kPaso];

    parsimonia.set_dtk( kPaso );

    aCerebro.BackPropagation_init;
    aData:= trayectorias.GetPuntosOfkPaso( kPaso-1 );
    for kRec := 0 to high(aData) do
    begin
      aRec := aData[kRec];
      aRec.X.CopyToVectR( px );
      aCerebro.SetEntradas( px );
      aCerebro.evaluar;
      acum_e2 := acum_e2 + aCerebro.calc_err2(derr, aRec.CF )* fDivisor;
      aCerebro.BackPropagation_ClearErr;
      aCerebro.BackPropagation_addMuestra(derr);
    end;
    // como efecto secundario del for anterior, el Cerebro queda cargado
    // con las derivadas del error cuadrático del paso kPaso respecto de
    // los Parametros Reales (del cerebro del paso).
  end;
  Result := acum_e2;
end;


procedure TCF_fCerebros.acum_g(var grad: TVectR; const X: TVectR);
var
  aCerebro: TCerebro;

  cerebros: TDAOfCerebros;
  kPaso: integer;
  kpp: Integer;

  gpr_paso: TVectR;

  gprpp: TMatR;
  // ahora, se trata de recorrer nuevamente los pasos coleccionando
  // los gradientes del error cuadrático respecto de los Parametros de Parsimonia

begin
  cerebros:= CF.Cerebros;
  aCerebro:= Cerebros[1];

  grad.Ceros;

  gprpp:= TMatR.Create_Init( grad.n, aCerebro.DimParametros );
  gpr_paso:= TVectR.Create_INit( aCerebro.DimParametros );

  for kPaso:= 1 to high( Cerebros ) do
  begin
    aCerebro:= Cerebros[kPaso];
    parsimonia.set_dtk( kPaso );
    parsimonia.gp( gprpp );

    aCerebro.GetGradX( gpr_paso );
    // ahora calculamos el producto gprpp * gpr_paso y lo acumulamos
    // en grad
    for kpp:= 1 to grad.n do
      grad.acum_e( kpp, gprpp.Fila( kpp ).PEV( gpr_paso) );
  end;
end;


function EntrenarCerebrosConParsimonia(
   CF: TAdminEstadosCerebro;
   trayectorias: TTrayectorias;

  parsimonia: TParsimonia;

  MaxNIter: integer; // Número máximo de iteracciones
  out flgConvergio: boolean;
  // retorna TRUE si convergió, FALSE si salio por número máximo de iters
  out cntIters: integer;
  kCapa, kNeurona: integer): NReal;

var
  CFX_fEntrenador: TCF_fCerebros;
  nParametros: integer;
  X, Xs: TVectR;
  problema_opt: TProblema_m01;
  k: integer;
  resValCosto, resValLagrangiana: NReal;
  resErr: NReal;
  kOffset: integer;

begin

  CFX_fEntrenador := TCF_fCerebros.Create(CF, trayectorias, parsimonia );

  nparametros := parsimonia.dimParsimonia;

  X := TVectR.Create_init(nparametros);
  Xs := TVectR.Create_init(nparametros);

  //  writeln('Inicio Opt por Problema ... ', kCapa,' ', kNeurona );

  problema_opt := TProblema_m01.Create_init(1, nparametros + 1, 0, nil, nil);
  problema_opt.f := CFX_fEntrenador;
  for k := 1 to nparametros do
  begin
    problema_opt.cota_inf_set(k, -1E50);
    problema_opt.cota_sup_set(k, 1E50);
  end;

  resErr := problema_opt.MinInBox_CPX(X, 100, 1E-25, MaxNIter, cntIters,
    resValLagrangiana, resValCOsto, flgConvergio, False);



  CFX_fEntrenador.SetParametrosCerebros( X );

  X.Free;
  Xs.Free;
  CFX_fEntrenador.Free;
  Result := resValCosto;

end;



function AjusteCFCerebro(
   CF: TAdminEstadosCerebro;
   trayectorias: TTrayectorias; parsimonia: TParsimonia; learningRate: NReal ): NReal;
var
  kPaso, nPuntosT_Actualizables, cntIters: integer;
  aCerebro: TCerebro;
  eMaxAbs, aMaxAbs: NReal;
  parametros, parametros_x: TVectR;
  sal: textfile;
  flg_Convergio: boolean;
  parametros_CF: TDAOfVectR;

begin
  nPuntosT_Actualizables:=  CF.nPuntosT - 1;
  if nPuntosT_actualizables > length( trayectorias.Trayectorias[0].puntos )  then
    nPuntosT_actualizables:= length( trayectorias.Trayectorias[0].puntos ) ;

  aCerebro:= CF.Cerebros[1];

  parametros:= TVectR.Create_init( aCerebro.DimParametros );
  parametros_x:= TVectR.Create_init( aCerebro.DimParametros );
  eMaxAbs:= 0;

  assignfile( sal, 'parameros_cerebro_porpasos.xlt' );
  rewrite( sal );



  if Parsimonia = nil then
  begin
    for kPaso:= 1 to nPuntosT_Actualizables do
    begin
      aCerebro:= CF.Cerebros[kPaso];
      aCerebro.GetParametros( parametros );

      Entrenar( aCerebro, trayectorias, kPaso-1 );

      aCerebro.GetParametros( parametros_x );
      aMaxAbs:= Parametros.distancia( Parametros_x );
      if aMaxAbs > eMaxAbs then
        eMaxAbs:= aMaxAbs;
      Parametros.PorReal( 1-learningRate );
      Parametros.sumRPV( learningRate, Parametros_X);
      aCerebro.SetParametros( Parametros );
      writeln( sal, kPaso, #9, Parametros.serialize_data( #9 ) );
      writeln( kPaso );
    end;
  end
  else
  begin
    setlength( parametros_CF, nPuntosT_Actualizables+1 );
    for kPaso:= 1 to nPuntosT_Actualizables do
    begin
      aCerebro:= CF.Cerebros[kPaso];
      parametros_CF[kPaso]:= TVectR.Create_Init( aCerebro.DimParametros );
      aCerebro.GetParametros( parametros_CF[kPaso] );
    end;

    EntrenarCerebrosConParsimonia( CF, trayectorias, parsimonia, 1000,
     flg_COnvergio, cntIters, -1, -1 );

    for kPaso:= 1 to nPuntosT_Actualizables do
    begin
      aCerebro:= CF.Cerebros[kPaso];
      aCerebro.GetParametros( parametros_x );
      aMaxAbs:= parametros_CF[kPaso].distancia( Parametros_x );
      if aMaxAbs > eMaxAbs then
              eMaxAbs:= aMaxAbs;


      parametros_CF[kPaso].PorReal( 1-learningRate );
      parametros_CF[kPaso].sumRPV( learningRate, Parametros_X);
      aCerebro.SetParametros( parametros_CF[kPaso] );
      writeln( sal, kPaso, #9, parametros_CF[kPaso].serialize_data( #9 ) );
      writeln( kPaso );

      parametros_CF[kPaso].free;
    end;
    setlength( parametros_CF, 0 );
  end;


  closefile( sal );
  result:= eMaxAbs;
end;



end.

